
#include <hardware/gpio.h>
#include <hardware/pio.h>
#include <hardware/pwm.h>
#include <pico/time.h>

#include "tool78_defs.h"
#include "tool78_hw_helpers.h"

#include "tool78.pio.h"

#include "tool78_hw.h"

static struct tool78_pio_vars vars;

int t78k0_uart2_send(int len, const uint8_t* data, int32_t timeout_us);
void t78k0_uart2_deinit(void);
int t78k0_uart2_has_available(void);
int t78k0_uart2_recv(int len, uint8_t* data, int32_t timeout_us);
int t78k0_uart2_send(int len, const uint8_t* data, int32_t timeout_us);

static bool t78k0_uart2_extclk_init(void) {
	if (!tool78_hw_init_help(&tool78_uart_tx_program, &tool78_uart_rx_program,
				&vars)) return false;

	vars.exclusive = false;

	// extclk between 2 and 20 MHz, let's do 8 MHz
	pwm_config pc = pwm_get_default_config();
	pwm_config_set_clkdiv(&pc, (float)clock_get_hz(clk_sys) / 8e6);
	pwm_init(pwm_gpio_to_slice_num(PINOUT_TOOL78_78K0_CLOCK), &pc, true);
	gpio_set_function(PINOUT_TOOL78_78K0_CLOCK, GPIO_FUNC_PWM);

	tool78_entryseq_78k0(tool78k0_uart2_extclk);

	tool78_uart_tx_program_init(PINOUT_TOOL78_PIO, vars.smtx, vars.txoff,
			PINOUT_TOOL78_78K0_TXMOSI, 9600, true);
	tool78_uart_rx_program_init(PINOUT_TOOL78_PIO, vars.smrx, vars.rxoff,
			PINOUT_TOOL78_78K0_RXMISO, 9600, true);

	// magic initial not-really-commands
	// wait tCOM or tR1 (unclear) (>445k/8M s + (2^16)/2M s)
	// send 0x00
	// wait t12 (>15k/8M s)
	// send 0x00
	// wait t2C (>15k/8M s)

	busy_wait_us_32(477768); // tCOM or tR1

	uint8_t byte = 0x00;
	t78k0_uart2_send(1, &byte, -1);

	busy_wait_us_32(1900); // t12

	t78k0_uart2_send(1, &byte, -1);

	busy_wait_us_32(1900); // t2C

	// now a reset command needs to be sent, but we leave that to the upper
	// (command processing) layer as it has to know about those timings anyway

	return true; // all is well!
}
static void t78k0_uart2_extclk_deinit(void) {
	t78k0_uart2_deinit(); // sets extclk to no func

	pwm_set_enabled(pwm_gpio_to_slice_num(PINOUT_TOOL78_78K0_CLOCK), false);
}

struct tool78_hw tool78_hw_78k0_uart2_extclk = {
	.target = tool78k0_uart2_extclk,

	.init = t78k0_uart2_extclk_init,
	.deinit = t78k0_uart2_extclk_deinit,

	.has_available = t78k0_uart2_has_available,

	.recv = t78k0_uart2_recv,
	.send = t78k0_uart2_send
};

